@@ -10,6 +10,7 @@ from aiotfm.utils import Locale, get_keys, shakikoo
 from aiotfm.connection import Connection
 from aiotfm.player import Profile, Player
 from aiotfm.tribe import Tribe
+from aiotfm.friend import Friend
 from aiotfm.message import Message, Whisper, Channel, ChannelMessage
 from aiotfm.shop import Shop
 from aiotfm.inventory import Inventory, InventoryItem, Trade
@@ -77,6 +78,7 @@ class Client:
 		self.auto_restart = auto_restart
 		self.api_tfmid = None
 		self.api_token = None
+		self._restarting = False
 
 		self._channels = []
 
@@ -171,11 +173,22 @@ class Client:
 			# :param *args: a list of string used as replacement inside the message.
 			self.dispatch('server_message', self.locale[t_key], *t_args)
 
+		elif CCC == (8, 1): # Play emote
+			player = self.room.get_player(pid=packet.read32())
+			emote = packet.read8()
+			flag = packet.readUTF() if emote == 10 else ''
+
+			# :desc: Called when a player plays an emote.
+			# :param player: :class:`aiotfm.Player` the player.
+			# :param emote: :class:`int` the emote's id.
+			# :param flag: :class:`str` the flag's id.
+			self.dispatch('emote', player, emote, flag)
+
 		elif CCC == (8, 5): # Show emoji
 			player = self.room.get_player(pid=packet.read32())
 			emoji = packet.read8()
 
-			# :desc: Called a player is showing an emoji above its head.
+			# :desc: Called when a player is showing an emoji above its head.
 			# :param player: :class:`aiotfm.Player` the player.
 			# :param emoji: :class:`int` the emoji's id.
 			self.dispatch('emoji', player, emoji)
@@ -187,7 +200,7 @@ class Client:
 			order = packet.read8()
 			player_time = packet.read16() / 100
 
-			# :desc: Called a player get the cheese to the hole.
+			# :desc: Called when a player get the cheese to the hole.
 			# :param player: :class:`aiotfm.Player` the player.
 			# :param order: :class:`int` the order of the player in the hole.
 			# :param player_time: :class:`float` player's time in the hole in seconds.
@@ -425,7 +438,7 @@ class Client:
 
 				# :desc: Called when the client receives the result of leaving a channel.
 				# :param result: :class:`int` result code.
-				self.dispatch('channel_leaved_result', result)
+				self.dispatch('channel_left_result', result)
 
 			elif TC == 59: # Channel /who result
 				idSequence = packet.read32()
@@ -844,8 +857,13 @@ class Client:
 		:param delay: :class:`int` the delay before restarting. Default is 5 seconds.
 		:param args: arguments to pass to the :meth:`Client.restart` method.
 		:param kwargs: keyword arguments to pass to the :meth:`Client.restart` method."""
+		if self._restarting:
+			return
+
+		self._restarting = True
 		await asyncio.sleep(delay)
 		await self.restart(*args, **kwargs)
+		self._restarting = False
 
 	async def restart(self, keys=None):
 		"""Restarts the client.
@@ -857,6 +875,10 @@ class Client:
 
 		self.close()
 
+		# If we don't recreate the connection, we won't be able to connect.
+		self.main = Connection('main', self, self.loop)
+		self.bulle = None
+
 		if keys is not None:
 			self.keys = keys
 		else:
@@ -996,6 +1018,22 @@ class Client:
 			await asyncio.sleep(1)
 			await self.whisper(username, message[i:i + 255])
 
+	async def getFriendList(self):
+		"""|coro|
+		Get the client's friend list
+
+		:return: :class:`list`
+			List of friends
+		"""
+		await self.sendCP(28)
+
+		def is_friend_list(tc, packet):
+			return (tc == 34)
+
+		tc, packet = await self.wait_for('on_raw_cp', is_friend_list)
+
+		return Friend.from_packet(packet)
+
 	async def getTribe(self, disconnected=True):
 		"""|coro|
 		Gets the client's :class:`aiotfm.Tribe` and return it
